@@ -5,7 +5,8 @@
 JHUurl="https://raw.githubusercontent.com/CSSEGISandData/COVID-19/master/csse_covid_19_data/"
 TSdomain="csse_covid_19_time_series"
 AGGdomain="csse_covid_19_daily_reports"
-TSfiles="time_series_covid19_confirmed_global.csv time_series_covid19_deaths_global.csv time_series_covid19_recovered_global.csv"
+TSfiles="time_series_covid19_confirmed_global.csv time_series_covid19_deaths_global.csv time_series_covid19_recovered_global.csv 
+	time_series_covid19_confirmed_US.csv time_series_covid19_deaths_US.csv"
 
 ### Time Series data
 for file in `echo ${TSfiles}`; do
